#!/usr/bin/python

program_name = "pycfg"
program_version = "0.2-pre1"

"""
PyCFG

Copyright (C) 2005 Scott Scriven

Released under the terms of the GNU Public License
"""

class config:
	def __init__(self, name=program_name):
		import os
		try:
			home = os.environ["HOME"]
		except:
			home = "/tmp"

		self._name = name
		self._filenames = [
				"/etc/%s/%s.conf" % (self._name, self._name),
				"%s/.%s.conf" % (home, self._name),
				"%s/.%src" % (home, self._name),
				"%s/.%s/rc" % (home, self._name),
				"%s/.%s/%s.conf" % (home, self._name, self._name),
				]
		self._namespace = {}

	def default(self, **args):
		for k in args.keys():
			v = args[k]
			self._namespace[k] = v

	def load(self):
		import os.path
		for name in self._filenames:
			if os.path.exists(name):
				self.parse(name)

		for k in self._namespace.keys():
			if k[0] != "_":
				setattr(self, k, self._namespace[k])

	def parse(self, filename):
		#print "parsing %s..." % filename
		try:
			execfile(filename, self._namespace)
		except:
			print "Error in config file %s" % filename
			import traceback, sys
			foo = traceback.format_exception(sys.exc_type,
					sys.exc_value,
					sys.exc_traceback)
			for item in foo:
				item = item[:-1]
				print item
			print "Press Enter to continue..."
			raw_input()

	def show(self):
		print str(self)

	def __str__(self):
		text = ""
		for k in self._namespace.keys():
			if k[0] != "_":
				text = text + "%s = %s\n" % (k, repr(self._namespace[k]))
		text = text[:-1] # chop trailing \n
		return text

	def edit_gui(self):
		import gtk
		self._build_gui()
		self._populate_gui()
		gtk.mainloop()

	def _build_gui(self):
		import gtk
		w = gtk.Window()
		w.set_title("PyCFG GUI: %s" % (self._name))
		w.connect("destroy", self._destroy_win)
		w.set_default_size(400, 400)
		self._win = w

		vb = gtk.VBox()
		w.add(vb)
		self._vb = vb

		self._build_menus()

		sw = gtk.ScrolledWindow()
		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self._sw = sw
		vb.add(sw)

		t = gtk.Table()
		self._values = t
		sw.add_with_viewport(t)

		ab = gtk.Button("Add")
		ke = gtk.Entry()
		ve = gtk.Entry()
		ke.set_text("name")
		ve.set_text("value")
		t.attach(ke, 0,1,65530,65531, 0, 0)
		t.attach(ve, 1,2,65530,65531, 0, 0)
		t.attach(ab, 2,3,65530,65531, 0, 0)

		l = gtk.Label("Note: this program does not actually work yet")
		vb.pack_start(l, 0, 0)

		w.show_all()

	def _build_menus(self):
		import gtk
		ag = gtk.AccelGroup()
		self._win.add_accel_group(ag)
		itemf = gtk.ItemFactory(gtk.MenuBar, "<main>", ag)
		itemf.create_items([
				("/File", None, None, 0, "<Branch>"),
				("/File/_Save", "<Control>S", None, 0, ""),
				("/File/Save _As", "<Control>A", None, 0, ""),
				("/File/_Quit", "<Control>Q", self._destroy_win, 0, ""),
				("/Help", None, None, 0, "<LastBranch>"),
				("/Help/About", None, None, 0, ""),
				])
		mb = itemf.get_widget("<main>")
		self._vb.pack_start(mb, 0, 1)
		self._menubar = mb

	def _populate_gui(self):
		import gtk
		keys = self._namespace.keys()
		keys.sort()
		row = 0
		for k in keys:
			if k[:1] == "_": continue
			v = self._namespace[k]
			if isinstance(v, type(gtk)): continue
			# TODO: handle each data type specially
			#print "%s: %s" % (k,v)
			row += 1
			b = gtk.Button("del")
			l = gtk.Label(k)
			e = gtk.Entry()
			e.set_text(repr(v))
			self._values.attach(l, 0, 1, row, row+1, 0, 0)
			self._values.attach(e, 1, 2, row, row+1, 0, 0)
			self._values.attach(b, 2, 3, row, row+1, 0, 0)
		self._win.show_all()

	def _destroy_win(self, *args):
		import gtk
		self._win.unrealize()
		while gtk.events_pending(): gtk.mainiteration()
		gtk.mainquit()

def run(args):
	import sys
	if len(sys.argv) > 1:
		programs = sys.argv[1:]
	else:
		programs = [program_name]
	for p in programs:
		cfg = config(p)
		cfg.load()
		#cfg.show()
		cfg.edit_gui()

if __name__ == "__main__":
	import sys
	run(sys.argv[1:])
